Authoring & Building Tcl Extensions with Teaish
========================================================================

<!-- Maintenance reminder: use _ instead of * for italics and bold because
emacs's markdown-mode gets painfully slow on moderately-sized docs which
use much of the latter.  https://emacs.stackexchange.com/a/63979/41203 -->

<div class='hidden'>If you are reading this from a local copy, as
opposed to from a Fossil web interface, many of the content links will
not work, as they refer to in-repository content.</div>

- [Creating a New Extension](#create)
  - [Including Teaish in the Extension](#create.full)
- [Invoking Extension Builds](#build)
- [(Un)Installing Extension](#install)
- [Extension Definition Files](#files)
  - [Core extension definition (`teaish.tcl`)](#tcl)
     - [`teaish.config` as an alternative to `teaish.tcl`](#teaish.config)
  - [Building (`teaish.make.in`)](#make)
     - [Hook into various build phases](#hooks)
  - [Add custom Tcl init code (`teaish.pkginit.tcl`)](#pkginit)
  - [Customize package loading (`pkgIndex.tcl.in`)](#pkgindex)
  - [Test (`teaish.test.tcl`)](#test)
- [Distributing Extensions](#dist)
  - [Including Teaish in the Dist](#dist.full)
- [Template File Filtering](#filtering)
- [Script-only Extensions](#scriptonly)
- [API Docs](#apidoc)

Intro
========================================================================

This page covers the build-related aspects of Tcl extensions, as they
pertain to teaish. It does not cover the C-related details of creating
Tcl extensions.  For example purposes this document uses the SQLite
Tcl extension because it's readily handy and was the initial
motiviation for this project.

Having at least a basic working knowledge of [Autosetup][] will
be helpful for using teaish.

Creating a teaish build requires some combination of the following
files:

- [`teaish.tcl`](#tcl) (required, though it may be empty for
  extremely basic cases)\  
  This is loaded by the teaish [Autosetup][] module to incorporate an
  extension's customizations, e.g. `--configure-flags`, into the
  build.

- [`teaish.make.in`](#make) (optional) is a makefile stub which
  gets `include`d by the teaish-generated Makefile. It needs to define
  certain variables so that the extension-agnostic part of the build
  can know how to build the extension.

- [`teaish.test.tcl.in` or `teaish.test.tcl.in`](#test) (optional)\  
  Can be used to create tests for the extension, such that `make test`
  will "just work."

- [`teaish.pkginit.tcl.in` or `teaish.pkginit.tcl`](#pkginit) (optional)\  
  An optional script of code to be run immediately after the module is
  loaded using either [the `make test` rules](#test) or the default
  `pkgIndex.tcl`.

- [`pkgIndex.tcl.in` or `pkgIndex.tcl`](#pkgindex) (optional)\  
  A custom `pkgIndex.tcl` for the package. The default is suitable
  for most cases.

These are covered in more detail in their own sections below, but see
the next section for how to quickly generate new ones.

Example builds:

- [Hello, World](/dir/teaish/example/hello) is a hello-world
  extension taken from the Tcl wiki.
- [SQLite3](/dir/teaish/example/sqlite) with many
  extension-specific configuration options.
- [Also SQLite3](/dir/teaish/example/sqlite-alt) with most config choices
  made for the user (based off of historical SQLite TEA configurations).
- [TclTLS](/dir/teaish/example/tcltls) is a proof-of-concept
  build which uses the sources from a checked-out copy of
  [TclTLS](https://core.tcl-lang.org/tcltls).
- [whutil](/dir/teaish/example/whutil) and
  [tmplish](/dir/teaish/example/tmplish) are script-only
  extensions.

<a name="create"></a>
Creating a New Extension
========================================================================

The easiest way to create a new set of teaish build files is:

>
```
$ /path/to/teaish/configure \
  --teaish-create-extension=/output/dir
```

Which will create the output dir (if needed) and write the following files to it:

- [teaish.tcl][]
- [teaish.make.in][]
- [teaish.test.tcl][]
- `teaish.c`: a hello-world-style extension

If all goes well then it should build as-is with:

>
```
$ cd the/output/dir
$ /path/to/teaish/configure \
  [--with-tcl=/path/to/tcl/install/prefix] # if needed
$ make test
```

And [(un)installed](#install) with:

>
```
$ make install   # will run an additional test
$ make uninstall # in case of later regret!
```

It can be [packaged up](#dist) for distribution with:

>
```
$ make dist
```

The dist files are not cleaned up by `make clean`, but
`make undist` will remove them.

<a name="create.full"></a>
Including Teaish in the Extension
------------------------------------------------------------------------

A full copy of teaish and Autosetup can be installed directly to an
existing extension's directory with:

>
```
$ /path/to/configure --teaish-install=/path/to/extension
```

When using `--teaish-create-extension`, teaish can be included by
adding the `--teaish-install` flag with no value or with the value
`auto`.

With that in place, the extension can be built using the conventional:

>
```
$ ./configure && make
```

To install a newer version of teaish over an existing copy in an
extension, use:

>
```
$ /path/to/new/teaish/configure \
    --teaish-install=/path/to/extension \
    --teaish-force
```

But _ACHTUNG_: that will overwrite all files in the target directory
which teaish believes it owns, including: `configure`, `auto.def`,
`pkgIndex.tcl.in`, `_teaish.tester.tcl.in`[^uscore], `Makefile.in`.

<a name="build"></a>
Invoking Extension Builds
========================================================================

There are three ways to plug an extension into this build system:

1. Place a copy of the the extension's teaish files in the main teaish
   directory and run `./configure`.

2. From a directory containing the extension's teaish files
   run `/path/to/teaish/configure`.

3. From a completely unrelated directory run\  
   `/path/to/teaish/configure --teaish-extension-dir=/path/to/extension ...`

(Pass `--help` to the `configure` to see the list of options.)

After that, just type `make test` and (if all goes well)...

<a name="install"></a>
(Un)Installing Extensions
========================================================================

Simply:

>
```
$ make install
```

That will build the extension, if needed, but it won't implicitly run
`make test`.

Immediately after the installation, the makefile will run a "package
require" test on the extension to ensure that the newly-installed
extension can be loaded that way. It will complain loudly if that
part fails.

> Tip: the first-ever failure of such a test was caused by the
  `Tcl_PkgProvides()` call in the extension's C file using the wrong
  case for the teaish [`-name.pkg`](#pkginfo). If your test fails
  without much useful information, check that first.

To uninstall:

>
```
$ make uninstall
```

That does nothing more than delete the directory into which the
extension is installed. (_Presumably_ only that extension is installed
in it.)

<a name="files"></a>
Extension Definition Files
========================================================================

This section covers the various files an extension may include to
influence aspects of its build or execution.


<a name="tcl"></a><a name="teaish.tcl"></a>
`teaish.tcl`
------------------------------------------------------------------------

This script conceptually needs to do several things, though all of
them have usable (though perhaps not useful) defaults.

This script is loaded by teaish very early on in the bootstrapping
process so that the extension may extend the set of `--flags`
available to the configure script. That enables the extension to
influence, for example, the `--help` text.

When the script is initially loaded, it ___must not invoke any
teaish-defined APIs___, as the configuration API is not yet fully
bootstrapped when the extension script is initially read.  Any
exceptions to this rule in the teaish API are clearly noted in their
API docs.

However, the script _is_ expected to run tell teaish a little bit about
the extension by calling `teaish-pkinfo-set` with a set of flags, like:

<a name="pkginfo"></a>

>
```
teaish-pkginfo-set -name foo -version 0.0.1
```

Or its equivalent:

>
```
teaish-pkginfo-set {-name foo -version 0.0.1}
```

(The latter does not require backslash-escaping for multi-line
invocations.)

It has a number of flags, all of which have defaults.

- __`-name name`__\  
  The extension's name. It defaults to the name of the directory
  containing the extension. (In TEA this would be the `PACKAGE_NAME`,
  not to be confused with...)
  - __`-name.pkg pkg-provide-name`__\  
    The extension's name for purposes of `Tcl_PkgProvide()`,
    `package require`, and friends. It defaults to the `-name`, and is normally
    the same, but some projects (like SQLite) have a different name here
    than they do in their historical TEA `PACKAGE_NAME`.
  - __`-name.dist name`__\  
    Is intended to be the base name for ["dist" bundles](#dist), to which
    the version number gets appended. Defaults to `-name`.
- __`-version the-version`__\  
  The extension's package version. Defaults to 0.0.0.
- __`-libDir dirname`__\  
  The base name of the directory into which this extension should be
  installed. It defaults to a concatenation of `-pkgname` and
  `-version`.
- __`-loadPrefix symbol`__\  
  For use as the second argument passed to [Tcl's `load`
  command][tcl-load] in the package-loading process. It defaults to
  title-cased `-name.pkg` because Tcl's `load` plugin system expects it
  in that form.
- __`-vsatisfies {...}`__\  
  Expects a list-of-lists of conditions for Tcl's `package vsatisfies`
  command: each list entry is a sub-list of `{PkgName Condition...}`.
  Teaish inserts those checks via its default
  [`pkgIndex.tcl.in`](#pkgindex) and [test-runner](#test) templates to
  verify that the system's package dependencies meet these
  requirements. The default value is `{{Tcl 8.5-}}` (recall that it's
  a list-of-lists), as 8.5 is the minimum Tcl version teaish will run
  on, but some extensions may require newer versions or dependencies
  on other packages. As a special case, if `-vsatisfies` is given a
  single token, e.g. `8.6-` then it is transformed into `{Tcl $thatToken}`,
  i.e. it checks the Tcl version which the package is being run with.
  If given multiple lists, each `package provides` check is run in the
  given order. Failure to meet a `vsatisfies` condition triggers an
  error.
- __`-pkgInit.tcl filename`__\
  The default test script and `pkgIndex.tcl` will automatically load
  the given file. Its name must be relative to the top of
  the extension's dir.\  
    _OR_...
    - __`-pkgInit.tcl.in filename.in`__\
      The default test script and `pkgIndex.tcl` will [filter](#filtering)
      this file to create `filename` (with its extension stripped) in the
      build directory, which will then use it as if it had been passed
      to `-pkgInit.tcl`. Cannot be used together with `-pkgInit.tcl`.
- __`-tm.tcl filename`__\  
  Specifies that the module is a [tcl::tm][]-style module, composed of
  only a single script with the given name. This automatically clears
  any `pkgIndex` state and enables the `no-dll` pragma (see
  below). The [installation rules](#install) will deploy the file
  renamed to
  `pkgName-version.tm`. \  
    _OR_...
    - __`tm.tcl.in filename.in`__\
    Works like `-tm.tcl` but [filters](#filtering) the given file to
    generate the same name without its extension. (This form is useful
    if you would like to rely on the filtering to keep the
    `-version` in sync between `teaish.tcl` and the library:
    use the `@TEAISH_VERSION@` [filter tag](#filtering) for that.)
- __`-options {...}`__\  
  If provided, it must be a list compatible with Autosetup's
  `options-add` function. These can also be set up via
  `teaish-options`, described below.
- __`-src {...}`__\
  A list of source code files with paths relative to the extension's
  top-most directory. This is an alternative to later passing each
  one to `teaish-src-add -dir -dist`.
- __`-pragmas {...}`__\  
  A list of infrequently-needed lower-level directives which can
  influence teaish, including:
   - __`static-pkgIndex.tcl`__ tells teaish that the client manages their
     own [`pkgIndex.tcl`](#pkgindex), so that teaish won't try to
     overwrite it using a template.
   - __`no-dist`__ tells teaish to elide the [`make dist`](#dist)
     recipe from the makefile so that the client can implement it.
   - __`no-full-dist`__ tells teaish that the[`make dist`](#dist)
     rules should only package the extension, not a copy of teaish,
     even if teaish believes it should package them together
     ([details](#dist.full)).
     Similarly, __`full-dist`__ tells teaish to always include
     a full copy of teaish in the dist archives, so long as `no-dist`
     is not active _and_ the dist build is not run out-of-tree.
   - __`no-dll`__ tells teaish to elide the makefile rules which build the
     DLL, as well as any [test script](#test) and
     [pkgIndex.tcl](#pkgindex) references to them. The intent here is
     to (A) support client-defined build rules for the DLL and (B)
     eventually support script-only extensions.
   - __`no-vsatisfies-error`__ indicates that the generated test and
     pkgIndex.tcl should not treat a `-vsatisfies` test failure as
     fatal, and should instead simply return.
   - __`no-tester`__: disables automatic generation of
     [`teaish.test.tcl`](#test) even if a copy of
     `_teaish.tester.tcl.in` is found.
   - __`no-install`__ tells teaish to elide the [`make install`](#install)
     recipe from the makefile.

`teaish-pkginfo-set` may be called multiple times to set some options
in later calls, but it must be called (if at all) by the time
`teaish-options` (see below) returns. Calling it from
`teaish-configure` (see below) may confuse the configuration process.

Next, it _may_ define the following functions to influence the
downstream configure process...

- __`teaish-options`__\  
  Gets called very early in the configure process, before the
  validation of the configure-time `--flags`. Must return an empty
  string or a list in the format used by the [Autosetup][]'s `options`
  and `options-add` functions. This lets the extension extend the
  supported `--flags` for the configure process, including showing
  them in the `--help` output. The flags must not collide with those
  defined by Autosetup or teaish - if they do then bootstrapping of
  teaish will fail with an error to the effect that the offending flag
  has already been defined.  Autosetup's built-in `[options-add]` can
  be used to conditionally add new `--flags` from within this
  function - they will all be collapsed into a single list when it's
  time to process the `--flags`.
- __`teaish-configure`__\  
  This gets called after the configuration process is bootstrapped and
  is where the extension may perform arbitrary work to configure the
  build, e.g. by processing `--flags`, performing feature detection,
  etc. It has access to the full Autosetup API, plus the APIs defined
  by [teaish/core.tcl][], [proj.tcl][], and any Autosetup modules it loads
  with the `use` command. After `teaish-configure` completes,
  teaish will wrap up the configure process and write out any pending
  generated files. If `teaish-configure` generates an error,
  configuration will (unsurprisingly) fail.

To reiterate:

> Aside from where specifically noted, most teaish Tcl APIs (named
  `teaish-*`) are not valid to call until `teaish-configure` is
  called, at which point teaish has completed
  bootstrapping. `teaish-options` implementations must avoid the
  temptation to call arbitrary `teaish-*` APIs, deferring most until
  `teaish-configure`.
>
> The _only_ `teaish-*` APIs which is legal to call from the global
  scope, when the script is initially imported, before
  `teaish-options` is called, are:
>
  - `teaish-get` with a subset of its flags
  - `teaish-pkginfo-set` (then...)
  - `teaish-pkginfo-get`
  - `teaish-get` with its full set of flags
>
> Autosetup's facilities may be used, insofar as they don't interefere
  with teaish's bootstrapping, but all feature-checking "really
  should" be deferred until `teaish-configure` is called.


See [the hello example extension][hello.tcl] for examples.

 <a name="teaish.config"></a>
### `teaish.config`

For very simple cases, in particular [script-only
extensions](#scriptonly), [teaish.tcl][] may be replaced by a file
named `teaish.config`. Its only contents must be arguments for
[`teaish-pkginfo-set`](#pkginfo). For example:

>
```
{
  -name whutil
  -version 0.1
  -tm.tcl whutil.tcl
}
```

The contents of this file are read verbatim and treated as a Tcl list
which gets passed to `teaish-pkginfo-set`. The contents are _not_
`eval`'d.

If both [teaish.tcl][] and `teaish.config` exist, which one is chosen
is not specified. Use one or the other, but not both.


<a name="make"></a><a name="teaish.make.in"></a>
`teaish.make.in`
------------------------------------------------------------------------

This _optional_ makefile snippet gets `include`d by the main Makefile
and is intended to communicate info about how to build the
extension.

This file is optional because the same information can be provided
programmatically via `teaish-configure` (from [`teaish.tcl`](#tcl)),
but a makefile may be more convenient for some cases. It's also
possible to programmatically generate arbitrary makefile code and
inject that into the generated makefile instead of, or in addition to,
`teaish.make.in` using `teash-make-add` and friends.

See [](#filtering) for info on how makefiles get filtered.

At its simplest, this file looks something like:

>
```
# List of C source and/or object files:
tx.src = $(tx.dir)/src/tclsqlite3.c

# List of files to include with "make dist". Must be relative to
# the extension's dir.
tx.dist.files += src/tclsqlite3.c

# List of LDFLAGS:
tx.LDFLAGS  = @LDFLAGS_MATH@ @LDFLAGS_ICU@ @LDFLAGS_RT@ @LDFLAGS_ZLIB@

# List of CFLAGS/CPPFLAGS:
tx.CFLAGS   = @CFLAGS_ICU@ @OPT_FEATURE_FLAGS@
```

The main vars are:

- __`tx.dir`__ is the absolute path to the directory which holds the
  extension's build files.
- __`tx.src`__ should be assigned the source file(s) to compile. It
  can also be assigned object files.  They should generally be
  prefixed with `$(tx.dir)` so that they can be found by out-of-tree
  build. To set these from __`teaish-configure`__, use the
  `teaish-src-add` function.\  
  (C++ files are untested here, but might work if they're _not_ mixed
  together with C files.)
- __`tx.CFLAGS`__ should be populated to set any `CFLAGS` or `CPPFLAGS`
  flags which are required for compiling this extension. To set these from
  `teaish-configure`, use the `teaish-cflags-add` function.
- __`tx.LDFLAGS`__ should be assigned any linker flags.  To
  set these from `teaish-configure`, use the
  `teaish-add-ldflags` or `teaish-ldflags-prepend` functions.

("tx" is short for "teaish extension.")

The complete list of available variables, and their documentation, are
in the generated `Makefile`, which gets filtered from
[](/file/teaish/Makefile.in).

Note that the top-level makefile _does not_ honor `CFLAGS`, `CPPFLAGS`
and `LDFLAGS` passed directly to `make`. Instead, they must be passed
to `configure`, and they'll get baked into the resulting makefile
under a different name. (This behavior was adopted after seeing it
used to good effect in other trees. It wasn't my idea, but it's a
useful one for configure-scripted projects.)

<a name="hooks"></a>
### Hooking into Various Build Phases


`teaish.make.in` may hook into the `clean`, `distclean`, `test`,
`install`, `uninstall`, and `dist` targets by using make dependencies,
as demonstrated below. Each of those targets is made up of layers of
targets following this pattern:

- __`target`__: does nothing but depends on...
- __`target-post`__: does nothing but depends on...
- __`target-core`__: holds the primary recipe. It depends on...
- __`target-pre`__:
   - __`clean`__, __`distclean`__, __`dist`__, and __`uninstall`__:
     `target-pre` does nothing but clients may make it depend on their
     own targets to run things before `target-core` is run.
   -   __`install`__ and __`test`__: depends on __`target-prepre`__
     and does nothing except to act as a hook for for clients.
      - __`install-prepre`__ and __`test-prepre`__: both depend on the
        extension DLL. In order for clients to ensure that their `pre`
        targets do not run in parallel with the DLL build when using
        `make -j#`, they should add a `install-pre:mytarget`
        dependency and a `mytarget:$(tx.dll)` dependency (likewise for
        `test-pre`).
- __`target-extension`__ is reserved for use by
  [teaish.make](#make).  It has no dependencies and nothing depends
  on it unless the extension defines any such relationships.

Put differently, the dependencies look like:

- `clean`, `distclean`, and `uninstall`: `target` → `target-post` → `target-core` →
  `target-pre`
- `test` and `install`: `target` → `target-post` → `target-core` →
  `target-pre` → `target-prepre`

In the simplest case, they can be hooked into like this:

>
```
clean: clean-extension
clean-extension:
    rm -f my-stuff.foo
```

The pattern is the same for each, but there's a caveat when hooking
into the `test` and `install` targets: one has to be careful to
account for parallel builds.

If running targets in parallel (using `make -j#`) is safe for one's
build then the pattern shown above is sufficient. If, however, their
order needs to be tightly controlled, follow this pattern instead:

>
```
test-post: test-extension
test-extension: test-core
    ...do stuff here...
```

That will ensure that the main tests (`test-core`) run before
`test-extension`, even when building in multi-process mode using
make's `-j#` flag.


<a name="pkginit"></a>
`teaish.pkginit.tcl`
------------------------------------------------------------------------

This optional script is intended to hold code which should be run
immediately after the extension library is loaded.

A different file can be used for this purpose by passing one of
`-pkgInit.tcl` or `-pkgInit.tcl.in` to [the pkginit
process](#pkginit).


If `teaish.pkginit.tcl.in` is found in the extension directory, it
is used to generate the `teaish.pkginit.tcl`, [filtering it the
same way makefiles are](#filtering). If not, a static
`teaish.pkginit.tcl` is used if it's available.

- During [`make install`](#install) `teaish.pkginit.tcl` will be
  copied to the extension's installation dir and the generated
  `pkgIndex.tcl` will `source` it after loading the extension.
- During [`make test`](#test) the test wrapper script will `source` it
  after loading the library, before `source`'ing `teaish.test.tcl`
  (if available).

In the context of the `source` calls, a scope-local var named `dir`
will be set which holds the name of the directory from which the file
is loaded. This is derived from conventional `pkgIndex.tcl` usage.


<a name="pkgindex"></a>
`pkgIndex.tcl.in`
------------------------------------------------------------------------

By default a `pkgIndex.tcl` will be generated for an extension based
on [teaish's primary pkgIndex
template](/file/teaish/pkgIndex.tcl.in). A package may override
that in one of two ways:

- Place a `pkgIndex.tcl.in` file in the extension's dir. It is
  [filtered](#filtering) to create `pkgIndex.tcl`.

- Manage a static/hand-written `pkgIndex.tcl` by doing _both_ of the
  following:
  - Place a `pkgIndex.tcl` file in the extension's dir.
  - Add the following flag to your [`teaish-pkginfo-set`](#pkginfo)
    call: `-pragmas {static-pkgIndex.tcl}`.

That combination is required to ensure that teaish will not overwrite
your `pkgIndex.tcl`. The separate step is required because teaish
cannot otherwise distinguish between a hand-written `pkgIndex.tcl`
and one generated in a prior build.


<a name="test"></a><a name="teaish.test.tcl"></a>
`teaish.test.tcl`
------------------------------------------------------------------------

The `configure` process, by default, generates a file called
`teaish.tester.tcl` in the build directory, generated from teaish's
`_teaish.tester.tcl.in`[^uscore] (if it exists). It acts as a wrapper for
running arbitrary tests on the extension. To disable this, add
`-pragmas {no-tester}` to the [package init](#pkginfo). When including
a whole copy of teaish into a custom extension's tree,
`_teaish.tester.tcl.in` can be deleted to disable this support.

The extension may optionally include _either_ a `teaish.test.tcl.in`
(subject to [filtering](#filter)) or a `_teaish.test.tcl`[^uscore]
(not filtered). In the former case `configure` will generate the
latter from the former.

The `make test` target invokes `_teaish.tester.tcl` with three arguments:

1. The absolute path of the module's DLL file.

2. The module's "load prefix", for use as the 2nd argument
   [to Tcl's `load` function][tcl-load].

3. The absolute path of [teaish/tester.tcl][], which provides a small
   library of routines which are helpful for running tests.

`_teaish.tester.tcl` will do the following before running the tests:

- Use `package vsatisfies` to ensure that the the currently-running
  Tcl version matches the project's [`-vsatisfies` flag](#tcl).
- Load the module's DLL, if it's not explicitly disabled.
- `source` [teaish/tester.tcl][].
- If the project includes a [`pkginit` file](#pkginit) then it
  `source`s that file.
- If the project includes a file named [`teaish.test.tcl`](#test) then:
     - _Possibly_ perform some other mysterious bookkeeping or
       bootstrapping.
     - `source` that file.
- If `_teaish.test.tcl` is not available then it simply emits a note
  that the module was successfully loaded.

Before `source`'ing the pkginit and test files, it sets a call-local
`dir` var to the name of the directory from which `teaish.pkginit.tcl`
resp.  `teaish.test.tcl` were loaded. (This convention is adopted from
Tcl's `package` command.)

Test script may run arbitrary Tcl code for testing. If the script
propagates an error, `make test` will fail.  [teaish/tester.tcl][]
(see above) provides a set of helpful routines for running basic
tests.

Here are some [example tests from the SQLite module](/file/teaish/example/sqlite/teaish.test.tcl):

>
```
test-expect 1.0-open {
  sqlite3 db :memory:
} {}

test-expect 1.1-select {
  db eval {select 'hi, world',1,2,3}
} {{hi, world} 1 2 3}

test-expect 99.0-db-close {db close} {}
```

Tests which need to read or write files need to take care to do so
from/to the proper directory so that they will work properly when
invoked via out-of-tree builds:

- The test script itself is available as `$::argv0` and its containing
  directory can be derived from that using `[file dirname $::argv0]`.
- Tcl's builtin `pwd` command will give the directory from which the
  test script was invoked, which will differ from the script's
  directory when doing an out-of-tree build.


<a name="dist"></a>
Distributing Extensions
========================================================================

Teaish provides basic support for generating a "dist" zip file and
tarball for extensions by simply running `make dist`. By default
the distribution includes _only_ the extension, but see
[the next section](#dist.full) for how to ensure that teaish itself
is bundled along with the extension.

These `dist` rules can be disabled by passing `disable-dist` in the
[`teaish-pkginfo-set -pragmas` list](#pkginfo) in which case the
`dist` recipe is left undefined so that the client makefile may define
it. When doing an out-of-tree build, the `dist` rules are
_unconditionally disabled_ because handling of the paths for archival
purposes becomes troublesome to do from Makefile code.

By default, teaish will automatically include its "own" files in
the distribution bundle if they are present.

In addition, it will include any files or directories which are either:

- Passed to `[teaish-dist-add]` or `[teaish-src-add -dist]` via
  `[teaish-configure]` (in `teaish.tcl`).
- _Appended_ to the makefile var `$(tx.dist.files)` via
  [`teaish.make.in`](#make). If `teaish.make.in` overwrites that that
  variable then the automatically-added files will _not_ be included.

Caveat(s):

- "Dist" file names _must_ be relative to, and live under, the
  extension dir.

If _both_ `zip` and `tar` binaries[^ziptar] are found at configure-time,
running `make dist` will create:

- `$NAME-$VERSION.zip`
- `$NAME-$VERSION.tar.gz`

(Noting that it assumes a `tar` which supports `czf`, which some older
`tar` implementations do not.)

Each archive has a top-level directory named `$NAME-$VERSION`.

The `clean` and `distclean` targets _do not_ delete those, but
the `undist` target will.

To build a freshly-unpackaged extension, do:

>
```
# If teaish is included in the bundle (see below):
$ ./configure
# Otherwise:
$ /path/to/teaish/configure ...
$ make
```

Alternately, `make dist` can be told to include a full copy of
teaish along with it...

[^ziptar]: The zip-based build actually uses tar to do some of the
  heavy lifting, so building the a zip bundle counterintuitively
  requires `tar`.

<a name="dist.full"></a>
Including Teaish in the Dist
------------------------------------------------------------------------

Whether or not teaish itself is included in the `make dist` result
is determined automatically:

- If teaish lives in the same directory as the extension, it will bundle them together
  by default.
- If they are separate, it won't, by default, but...

The [pkginfo pragmas](#pkginfo) `full-dist` and `no-full-dist` can be used
to tell teaish to include, or not, itself in the dist bundle.

When full-dist is on then `make dist` will include all of teaish in the
distribution, such that running `./configure` from that directory is all
that's needed to prepare it for building.


<a name="filtering"></a>
Template File Filtering
========================================================================

Several of the files mentioned in this doc are filtered in a manner
conventional to GNU Autotools. Specifically, the follow the rules for
Autosetup's `make-template` function, but the conventions are similar.
In this project, all such files have a suffix of `.in` (by convention,
not hard-coded limitation).

Input files may have any number of `@PLACEHOLDERS@` in them, which
will be filtered out during the configure process, each one mapped to
an Autosetup `define`, and the results will be written to some target
file (typically the same as the input's name minus any `".in"`
extension). *All such placeholders must be resolved*, using values
provided by Autosetup, teaish, or set in `teaish.tcl` with Autosetup's
`define` function. If any unresolved placeholders remain after the
filtering step, `configure` will fail[^validation-exception].

To generate the whole list of placeholders, pass
`--teaish-dump-defines` to any `configure` invocation. That will
generate a file named `config.defines.txt` containing all of the
exported flags in some legible form (but don't rely on its format).

Filering of such files happens very late in the `configure` process,
as it must happen after all `define`s are set up.

[^validation-exception]: Exception: commented-out lines in files named
  `*[Mm]ake*` are ignored _if_ the line starts with `#` (ignoring
  leading whitespace). Aside from that exception, the content is
  semantically opaque to the validation, so it has no notion of what a
  "comment" is.

<a name="scriptonly"></a>
Script-only Extensions
========================================================================

Script-only extensions are particularly simple to set up, and there
are two approaches. All that's needed is a [teaish.tcl][] and the
accompanying script.

First, using a `pkgIndex.tcl`-style:

>
```
teaish-pkginfo-set {
  -name tmplish
  -version 1.0
  -pkgInit.tcl tmplish.tcl
  -pragmas {no-dll}
}
```

Or a [tcl::tm][]-style:

>
```
teaish-pkginfo-set {
  -name tmplish
  -version 1.0
  -tm.tcl tmplish.tcl
}
```

Alternatively, use a [teaish.config][] file instead.

Optionally, add a [teaish.test.tcl][] script to test it with.

Examples of script-only extensions can be found in
[teaish/example/whutil](/dir/teaish/example/whutil) and
[teaish/example/tmplish](/dir/teaish/example/tmplish).


[tcl::tm]: https://wiki.tcl-lang.org/page/tm

<a name="apidoc"></a>
API Docs
========================================================================

The Tcl-level API docs are scattered around the following places:

- Teaish itself: [](/dir/teaish/autosetup/teaish)`/*.tcl`
- [proj.tcl][] holds the APIs named `proj-*` (lots of project-agnostic
  utility code for use with Autosetup).
- Autosetup is documented in the many files in [](/dir/teaish/autosetup/).

Running `./configure --reference` will format the many docs for
reading.  Search that for `teaish-` to find the Teaish APIs. By
Autosetup conventions, functions are documented adding comments with
`@the-function-name`, so searching for `@teaish-` in the Teaish core
code will reveal the public APIs.


[Autosetup]: https://msteveb.github.io/autosetup/
[tcl-load]: https://www.tcl-lang.org/man/tcl8.6/TclCmd/load.htm
[proj.tcl]: /file/teaish/autosetup/proj.tcl
[teaish/core.tcl]: /file/teaish/autosetup/teaish/core.tcl
[teaish/tester.tcl]: /file/teaish/autosetup/teaish/tester.tcl
[hello.tcl]: /file/teaish/example/hello/teaish.tcl
[Haiku]: https://www.haiku-os.org/
[teaish.tcl]: #tcl
[teaish.test.tcl]: #teaish.test.tcl
[teaish.make.in]: #teaish.make.in
[teaish.config]: #teaish.config
[^uscore]: The underscore in the name `_teaish.tester.tcl.in` is so that
         that rarely-edited file will stop interfering with tab
         completion. The underscore prefixes in other _generated_ `_teaish.*`
         files serve the same purpose.

